// Copyright (c) 2023 mStar
// 
// Licensed under the EUPL, Version 1.2
// 
// You may not use this work except in compliance with the Licence.
// You should have received a copy of the Licence along with this work. If not, see:
// <https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12>.
// See the Licence for the specific language governing permissions and limitations under the Licence.
// 

mod diesel_db;

use anyhow::{Error, bail};
use diesel::{prelude::*, pg::{PgConnection, Pg}};
use diesel_migrations::{EmbeddedMigrations, embed_migrations, MigrationHarness};


pub const POSTGRES_MIGRATIONS: EmbeddedMigrations = embed_migrations!("migrations/postgres");

pub fn establish_connection(db_url: &str) -> Result<PgConnection, Error> {
    //SqliteConnection::establish(db_url).unwrap_or_else(|e| panic!("Error connecting to {}: {}", db_url, e))
    let mut conn = PgConnection::establish(db_url)
        .unwrap_or_else(|e| panic!("Error connecting to {}: {}", db_url, e));
    run_migrations(&mut conn)?;
    //c.run_pending_migrations(MIGRATIONS)?;
    Ok(conn)
}

fn run_migrations(conn: &mut impl MigrationHarness<Pg>) -> Result<(), Error> {
    match conn.run_pending_migrations(POSTGRES_MIGRATIONS) {
        Ok(_) => Ok(()),
        Err(e) => bail!("Migrations failed. Error {}", e)
    }
}